<?php
//
// XI Core Component Functions
// Copyright (c) 2008-2018 Nagios Enterprises, LLC. All rights reserved.
//

include_once(dirname(__FILE__) . '/../componenthelper.inc.php');
include_once(dirname(__FILE__) . '/ajaxhelpers.inc.php');
include_once(dirname(__FILE__) . '/dashlets.inc.php');
include_once(dirname(__FILE__) . '/status-utils.inc.php');


xicore_component_init();


////////////////////////////////////////////////////////////////////////
// COMPONENT INIT FUNCTIONS
////////////////////////////////////////////////////////////////////////


function xicore_component_init()
{
    $name = "xicore";
    $args = array(
        COMPONENT_NAME => $name,
        COMPONENT_TITLE => _("Nagios XI Core Functions"),
        COMPONENT_AUTHOR => _("Nagios Enterprises, LLC"),
        COMPONENT_DESCRIPTION => _("Provides core functions and interface functionality for Nagios XI."),
        COMPONENT_COPYRIGHT => _("Copyright (c) 2009-2018 Nagios Enterprises"),
        COMPONENT_PROTECTED => true,
        COMPONENT_TYPE => COMPONENT_TYPE_CORE
    );
    register_component($name, $args);
}


////////////////////////////////////////////////////////////////////////
// EVENT HANDLER AND NOTIFICATION FUNCTIONS
////////////////////////////////////////////////////////////////////////


register_callback(CALLBACK_EVENT_PROCESSED, 'xicore_eventhandler');


/**
 * Set up event type handlers for Nagios Core events
 *
 * @param   int     $cbtype     Callback type
 * @param   array   $args       Arguments
 */
function xicore_eventhandler($cbtype, $args)
{
    switch ($args["event_type"]) {
        case EVENTTYPE_NOTIFICATION:
            xicore_handle_notification_event($args);
            break;
        default:
            break;
    }
}


/**
 * Handle Nagios Core notifications through XI
 *
 * @param   array   $args   Notification event arguments
 */
function xicore_handle_notification_event($args)
{
    $debug = get_option("enable_subsystem_logging", true);

    // Don't do the notification events if we are inactive
    if (!is_license_activated()) {
        if ($debug) {
            echo "ERROR: License is not activated.\n";
        }
        return;
    }

    // Use this for debug output in PHPmailer log
    $debugmsg = "";

    $meta = $args["event_meta"];
    $contact = $meta["contact"];
    $nt = $meta["notification-type"];

    // Find the XI user
    $user_id = get_user_id($contact);
    if ($user_id <= 0) {
        if ($debug) {
            echo "ERROR: Could not find user_id for contact '" . $contact . "'\n";
        }
        return;
    }

    if ($debug) {
        echo "Got XI user id for contact '" . $contact . "': $user_id\n";
    }

    // Set user id session variable - used later in date/time, preference, etc. functions
    if (!defined("NAGIOSXI_USER_ID")) {
        define("NAGIOSXI_USER_ID", $user_id);
    }

    // Bail if user has notifications disabled completely
    $notifications_enabled = get_user_meta($user_id, 'enable_notifications');
    if ($notifications_enabled != 1) {
        if ($debug) {
            echo "" . _('ERROR: User has (global) notifications disabled!') . "\n";
        }
        return;
    }

    // EMAIL NOTIFICATIONS
    $notify = get_user_meta($user_id, "notify_by_email");
    $email_plaintext = (bool) get_user_meta($user_id, "email_plaintext", 0);

    // Default Priority
    $priority = ($meta['type'] == "PROBLEM") ? 1 : 0;

    // Get email notification options for user
    $notify_host_recovery = get_user_meta($user_id, 'notify_host_recovery');
    $notify_host_down = get_user_meta($user_id, 'notify_host_down');
    $notify_host_unreachable = get_user_meta($user_id, 'notify_host_unreachable');
    $notify_host_flapping = get_user_meta($user_id, 'notify_host_flapping');
    $notify_host_downtime = get_user_meta($user_id, 'notify_host_downtime');
    $notify_host_acknowledgment = get_user_meta($user_id, 'notify_host_acknowledgment', 1);
    $notify_service_recovery = get_user_meta($user_id, 'notify_service_recovery');
    $notify_service_warning = get_user_meta($user_id, 'notify_service_warning');
    $notify_service_unknown = get_user_meta($user_id, 'notify_service_unknown');
    $notify_service_critical = get_user_meta($user_id, 'notify_service_critical');
    $notify_service_flapping = get_user_meta($user_id, 'notify_service_flapping');
    $notify_service_downtime = get_user_meta($user_id, 'notify_service_downtime');
    $notify_service_acknowledgment = get_user_meta($user_id, 'notify_service_acknowledgment', 1);

    // Priority settings
    $notify_host_recovery_priority = get_user_meta($user_id, 'notify_host_recovery_priority');
    $notify_host_down_priority = get_user_meta($user_id, 'notify_host_down_priority');
    $notify_host_unreachable_priority = get_user_meta($user_id, 'notify_host_unreachable_priority');
    $notify_host_flapping_priority = get_user_meta($user_id, 'notify_host_flapping_priority');
    $notify_host_downtime_priority = get_user_meta($user_id, 'notify_host_downtime_priority');
    $notify_host_acknowledgment_priority = get_user_meta($user_id, 'notify_host_acknowledgment_priority');
    $notify_service_recovery_priority = get_user_meta($user_id, 'notify_service_recovery_priority');
    $notify_service_warning_priority = get_user_meta($user_id, 'notify_service_warning_priority');
    $notify_service_unknown_priority = get_user_meta($user_id, 'notify_service_unknown_priority');
    $notify_service_critical_priority = get_user_meta($user_id, 'notify_service_critical_priority');
    $notify_service_flapping_priority = get_user_meta($user_id, 'notify_service_flapping_priority');
    $notify_service_downtime_priority = get_user_meta($user_id, 'notify_service_downtime_priority');
    $notify_service_acknowledgment_priority = get_user_meta($user_id, 'notify_service_acknowledgment_priority');

    // Service
    if ($nt == "service") {
        switch ($meta['type']) {

            case "PROBLEM":
                if (($notify_service_warning != 1) && ($meta['servicestateid'] == 1)) {
                    $notify = 0;
                } else if (($notify_service_critical != 1) && ($meta['servicestateid'] == 2)) {
                    $notify = 0;
                } else if (($notify_service_unknown != 1) && ($meta['servicestateid'] == 3)) {
                    $notify =0;
                }
                
                if (($notify_service_warning_priority != 1) && ($meta['servicestateid'] == 1)) {
                    $priority = 0;
                } else if (($notify_service_critical_priority != 1) && ($meta['servicestateid'] == 2)) {
                    $priority = 0;
                } else if (($notify_service_unknown_priority != 1) && ($meta['servicestateid'] == 3)) {
                    $priority = 0;
                }
                break;

            case "RECOVERY":
                if ($notify_service_recovery != 1) {
                    $notify = 0;
                }
                $priority = $notify_service_recovery_priority;
                break;

            case "ACKNOWLEDGEMENT":
                if ($notify_service_acknowledgment != 1) {
                    $notify = 0;
                }
                $priority = $notify_service_acknowledgment_priority;
                break;

            case "FLAPPINGSTART":
            case "FLAPPINGSTOP":
                if ($notify_service_flapping != 1) {
                    $notify = 0;
                }
                $priority = $notify_service_flapping_priority;
                break;

            case "DOWNTIMESTART":
            case "DOWNTIMECANCELLED":
            case "DOWNTIMEEND":
                if ($notify_service_downtime != 1) {
                    $notify = 0;
                }
                $priority = $notify_service_downtime_priority;
                break;

        }

    // Host
    } else {
        switch ($meta['type']) {

            case "PROBLEM":
                if (($notify_host_down != 1) && ($meta['hoststateid'] == 1)) {
                    $notify = 0;
                } else if (($notify_host_unreachable != 1) && ($meta['hoststateid'] == 2)) {
                    $notify = 0;
                }
                
                if (($notify_host_down_priority != 1) && ($meta['hoststateid'] == 1)) {
                    $priority = 0;
                } else if (($notify_host_unreachable_priority != 1) && ($meta['hoststateid'] == 2)) {
                    $priority = 0;
                }
                break;

            case "RECOVERY":
                if ($notify_host_recovery != 1) {
                    $notify = 0;
                }
                $priority = $notify_host_recovery_priority;
                break;

            case "ACKNOWLEDGEMENT":
                if ($notify_host_acknowledgment != 1) {
                    $notify = 0;
                }
                $priority = $notify_host_acknowledgment_priority;
                break;

            case "FLAPPINGSTART":
            case "FLAPPINGSTOP":
                if ($notify_host_flapping != 1) {
                    $notify = 0;
                }
                $priority = $notify_host_flapping_priority;
                break;

            case "DOWNTIMESTART":
            case "DOWNTIMECANCELLED":
            case "DOWNTIMEEND":
                if ($notify_host_downtime != 1) {
                    $notify = 0;
                }
                $priority = $notify_host_downtime_priority;
                break;

        }    
    }

    if ($notify == 1) {

        if ($debug) {
            echo _("An email notification will be sent") . "...\n\n";
        }

        // Get the user's email address
        $email = get_user_attr($user_id, "email");

        // Get the email subject and message
        if ($nt == "service") {
            $subject = get_user_service_email_notification_subject($user_id);
            $message = get_user_service_email_notification_message($user_id);
        } else {
            $subject = get_user_host_email_notification_subject($user_id);
            $message = get_user_host_email_notification_message($user_id);
        }

        $opts = array(
            "subject" => $subject,
            "message" => $message,
            "meta" => $meta
        );

        do_callbacks(CALLBACK_NOTIFICATION_TEMPLATE_EMAIL, $opts);

        // process the alert text and replace variables
        $subject = process_notification_text($opts["subject"], $opts["meta"]);
        $message = process_notification_text($opts["message"], $opts["meta"]);

        // Set where email is coming from for PHPmailer log
        $send_mail_referer = "includes/components/xicore/xicore.inc.php > Event Handler Notification Email";

        $opts = array(
            "referer" => $send_mail_referer,
            "to" => $email,
            "subject" => $subject,
            "high_priority" => $priority,
            "message" => $message,
            "plaintext" => $email_plaintext
        );

        do_callbacks(CALLBACK_NOTIFICATION_EMAIL, $opts);

        if ($debug) {
            echo "Email Notification Data:\n\n";
            print_r($opts);
            echo "\n\n";
        }

        send_email($opts, $debugmsg, $opts["referer"], $opts['plaintext']);

    } else {
        if ($debug) {
            echo "" . _('User has email notifications disabled.') . "\n\n";
        }
    }

    // MOBILE TEXT NOTIFICATIONS
    $notify = get_user_meta($user_id, "notify_by_mobiletext");
  
    // Get SMS notificaiton options for user 
    $notify_sms_host_recovery = get_user_meta($user_id, 'notify_sms_host_recovery', $notify_host_recovery);
    $notify_sms_host_down = get_user_meta($user_id, 'notify_sms_host_down', $notify_host_down);
    $notify_sms_host_unreachable = get_user_meta($user_id, 'notify_sms_host_unreachable', $notify_host_unreachable);
    $notify_sms_host_flapping = get_user_meta($user_id, 'notify_sms_host_flapping', $notify_host_flapping);
    $notify_sms_host_downtime = get_user_meta($user_id, 'notify_sms_host_downtime', $notify_host_downtime);
    $notify_sms_host_acknowledgment = get_user_meta($user_id, 'notify_sms_host_acknowledgment', $notify_host_acknowledgment);
    $notify_sms_service_recovery = get_user_meta($user_id, 'notify_sms_service_recovery', $notify_service_recovery);
    $notify_sms_service_warning = get_user_meta($user_id, 'notify_sms_service_warning', $notify_service_warning);
    $notify_sms_service_unknown = get_user_meta($user_id, 'notify_sms_service_unknown', $notify_service_unknown);
    $notify_sms_service_critical = get_user_meta($user_id, 'notify_sms_service_critical', $notify_service_critical);
    $notify_sms_service_flapping = get_user_meta($user_id, 'notify_sms_service_flapping', $notify_service_flapping);
    $notify_sms_service_downtime = get_user_meta($user_id, 'notify_sms_service_downtime', $notify_service_downtime);
    $notify_sms_service_acknowledgment = get_user_meta($user_id, 'notify_sms_service_acknowledgment', $notify_service_acknowledgment);
    
    // Service
    if ($nt == "service") {
        switch ($meta['type']) {

            case "PROBLEM":
                if (($notify_sms_service_warning != 1) && ($meta['servicestateid'] == 1)) {
                    $notify = 0;
                } else if (($notify_sms_service_critical != 1) && ($meta['servicestateid'] == 2)) {
                    $notify = 0;
                } else if (($notify_sms_service_unknown != 1) && ($meta['servicestateid'] == 3)) {
                    $notify = 0;
                }
                break;

            case "RECOVERY":
                if ($notify_sms_service_recovery != 1) {
                    $notify = 0;
                }
                break;

            case "ACKNOWLEDGEMENT":
                if ($notify_sms_service_acknowledgment != 1) {
                    $notify = 0;
                }
                break;

            case "FLAPPINGSTART":
            case "FLAPPINGSTOP":
                if ($notify_sms_service_flapping != 1) {
                    $notify = 0;
                }
                break;

            case "DOWNTIMESTART":
            case "DOWNTIMECANCELLED":
            case "DOWNTIMEEND":
                if ($notify_sms_service_downtime != 1) {
                    $notify = 0;
                }
                break;

        }

    // Host
    } else {
        switch ($meta['type']) {

            case "PROBLEM":
                if (($notify_sms_host_down != 1) && ($meta['hoststateid'] == 1)) {
                    $notify = 0;
                } else if (($notify_sms_host_unreachable != 1) && ($meta['hoststateid'] == 2)) {
                    $notify = 0;
                }
                break;

            case "RECOVERY":
                if ($notify_sms_host_recovery != 1) {
                    $notify = 0;
                }
                break;

            case "ACKNOWLEDGEMENT":
                if ($notify_sms_host_acknowledgment != 1) {
                    $notify = 0;
                }
                break;

            case "FLAPPINGSTART":
            case "FLAPPINGSTOP":
                if ($notify_sms_host_flapping != 1) {
                    $notify = 0;
                }
                break;

            case "DOWNTIMESTART":
            case "DOWNTIMECANCELLED":
            case "DOWNTIMEEND":
                if ($notify_sms_host_downtime != 1) {
                    $notify = 0;
                }
                break;
        }    
    }

    if ($notify == 1) {

        if (!is_user_phone_verified($user_id)) {
            if ($debug) {
                echo _('ERROR: User has enabled SMS notifications but phone has not been verified.') . "\n\n";
            }
            return;
        }

        if ($debug) {
            echo _('A mobile text notification will be sent...') . "\n\n";
        }

        // Get the user's mobile info
        $mobile_number = get_user_meta($user_id, "mobile_number");
        $mobile_provider = get_user_meta($user_id, "mobile_provider");

        // Generate the email address to use
        $email = get_mobile_text_email($mobile_number, $mobile_provider);

        if ($debug == true)
            echo "Mobile number: " . $mobile_number . ", Mobile provider: " . $mobile_provider . " => Mobile Email: " . $email . "\n\n";

        // Get the email subject and message
        if ($nt == "service") {
            $subject = get_user_service_mobiletext_notification_subject($user_id);
            $message = get_user_service_mobiletext_notification_message($user_id);
        } else {
            $subject = get_user_host_mobiletext_notification_subject($user_id);
            $message = get_user_host_mobiletext_notification_message($user_id);
        }

        $opts = array(
            "subject" => $subject,
            "message" => $message,
            "meta" => $meta
        );

        do_callbacks(CALLBACK_NOTIFICATION_TEMPLATE_SMS, $opts);

        // process the alert text and replace variables
        $subject = process_notification_text($opts["subject"], $opts["meta"]);
        $message = process_notification_text($opts["message"], $opts["meta"]);

        // Set where email is coming from for PHPmailer log
        $send_mail_referer = "includes/components/xicore/xicore.inc.php > Event Handler Notification Mobile Text";

        // Run bacllback on message
        $opts = array(
            "referer" => $send_mail_referer,
            "to" => $email,
            "subject" => $subject,
            "message" => $message
        );

        do_callbacks(CALLBACK_NOTIFICATION_SMS, $opts);

        if ($debug) {
            echo "Mobile Text Notification Data:\n\n";
            print_r($opts);
            echo "\n\n";
        }

        send_email($opts, $debugmsg, $opts["referer"], true);

    } else {
        if ($debug) {
            echo "User has mobile text notifications disabled.\n\n";
        }
    }

}

