#!/usr/bin/env php
<?php 

/* Usage: ./nxti_import.php <PATH>
 * Run this against a trap definition file (such that snmptt.conf) to parse 
 * data and insert it into the NXTI internal database. 
 */

require_once(dirname(__FILE__) . '/../html/includes/common.inc.php');
require_once(dirname(__FILE__) . '/../html/includes/components/nxti/includes/utils-traps.inc.php');

main();
exit();

function main() {
    global $argv;

    $filename = $argv[1];
    $contents = file($filename); // dont need to write

    $current_defn = array();

    $skip_until = -1;

    foreach ($contents as $ind => $line) {
        if ($ind <= $skip_until)
            continue; // between SDESC and EDESC
        $line = trim($line);

        switch ($line[0]) {
            case 'E':
                if ($line[1] === 'V') { // EVENT
                    db_insert($current_defn);
                    $current_defn = array();
                    // Initialize optional values
                    $current_defn['trapdata_desc'] = '';
                    $current_defn['trapdata_exec'] = array();
                    $current_defn['trapdata_raw_data'] = '';

                    parse_event($line, $current_defn);
                }
                else if ($line[1] === 'X') { // EXEC
                    parse_exec($line, $current_defn);
                }
                break;

            case 'F': // FORMAT
                parse_custom_format($line, $current_defn);
                break;

            case 'P': // PREEXEC
            case 'N': // NODES
            case 'M': // MATCH
            case 'R': // REGEX
                parse_raw_data($line, $current_defn);
                break;

            case 'S': // SDESC
                $skip_until = parse_description($contents, $ind, $current_defn);
                break;

            default:
                // Probably a pound comment line.
                break;
        }
    
    }
    db_insert($current_defn);

    $unused = '';
}

// EVENT name oid "c a t e g o r y" severity
function parse_event($line, &$defn) {
    $line = explode(' ', $line);
    $len = count($line);
    if ($len < 5)
        return;

    $defn['trapdata_event_name'] = $line[1];
    $defn['trapdata_event_oid'] = $line[2];
    $defn['trapdata_severity'] = $line[$len - 1];

    $tmp = array_slice($line, 3, count($line) - 4);
    $tmp = implode(" ", $tmp);
    $defn['trapdata_category'] = trim($tmp, '"');
}

function parse_exec($line, &$defn) {
    $line = explode(" ", $line);
    $line = array_slice($line, 1);
    $defn['trapdata_exec'][] = implode(" ", $line);
}

function parse_custom_format($line, &$defn) {
    $line = explode(" ", $line);
    $line = array_slice($line, 1);
    $defn['trapdata_custom_format'] = implode(" ", $line);

}

function parse_description($contents, $index, &$defn) {
    $defn['trapdata_desc'] = '';
    if (substr(trim($contents[$index]), 0, 5) === "SDESC") {
        foreach ($contents as $test_index => $line) {
            if ($test_index <= $index) {
                continue;
            }
            if (substr(trim($line), 0, 5) === "EDESC") {
                return $test_index;
            }
            else {
                $defn['trapdata_desc'] .= $line;
            }
        }
    }
    
}

function parse_raw_data($line, &$defn) {
    if (isset($defn['trapdata_raw_data'])) {
        $defn['trapdata_raw_data'] .= $line;
    }
    else {
        $defn['trapdata_raw_data'] = $line;
    }
}

function db_insert($trap_definition) {
    if ($trap_definition === array() || !isset($trap_definition['trapdata_event_name'])) {
        return FALSE;
    }
    //print_r($trap_definition);
    echo $trap_definition['trapdata_event_name'] . "\n";
    $existing = get_by_event_name_trapdata($trap_definition['trapdata_event_name']);
    if (is_array($existing) && isset($existing[0])) {
        // Update the trap definition rather than adding it
        $existing = $existing[0];

        // Assume that if other columns differ, the user prefers the one in our database.
        $affected_columns = array('trapdata_event_name',
                                  'trapdata_event_oid',
                                  'trapdata_category',
                                  'trapdata_severity',
                                  'trapdata_raw_data');
        foreach($affected_columns as $column) {
            if (!isset($trap_definition[$column])) {
                $trap_definition[$column] = $existing[$column];
            }
        }
        $result = update_trapdata_full($trap_definition['trapdata_event_name'],
                                      $trap_definition['trapdata_event_oid'],
                                      $trap_definition['trapdata_category'],
                                      $trap_definition['trapdata_severity'],
                                      $trap_definition['trapdata_raw_data']);
    }
    else {
        // New trap
        $result = add_trapdata($trap_definition['trapdata_event_name'], 
                               $trap_definition['trapdata_event_oid'], 
                               $trap_definition['trapdata_category'], 
                               $trap_definition['trapdata_severity'],
                               $trap_definition['trapdata_custom_format'],
                               $trap_definition['trapdata_raw_data'], 
                               $trap_definition['trapdata_exec'],
                               $trap_definition['trapdata_desc']);
    }

}

